{% load cms_tags %}

<div class="djangocms_faq_container">
    {% if instance.show_title %}
      <h1>{{ instance.title }}</h1>
    {% endif %}

    {% for plugin in instance.child_plugin_instances %}
        {% render_plugin plugin %}
    {% endfor %}
</div>
